﻿using Microsoft.Win32;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace Graphs
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public delegate double FuncDelegate(double x);
        private static FuncDelegate sinDelegate = Math.Sin,
                                    cosDelegate = Math.Cos;


        public int CoefAProperty
        {
            get { return (int)GetValue(CoefAPropertyProperty); }
            set { SetValue(CoefAPropertyProperty, value); }
        }

        // Using a DependencyProperty as the backing store for CoefAProperty.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty CoefAPropertyProperty =
            DependencyProperty.Register("CoefAProperty", typeof(int), typeof(MainWindow), new PropertyMetadata(0,PropertyChanged));



        public int CoefBProperty
        {
            get { return (int)GetValue(CoefBPropertyProperty); }
            set { SetValue(CoefBPropertyProperty, value); }
        }

        // Using a DependencyProperty as the backing store for CoefBProperty.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty CoefBPropertyProperty =
            DependencyProperty.Register("CoefBProperty", typeof(int), typeof(MainWindow), new PropertyMetadata(0,PropertyChanged));



        public FuncDelegate FunctionProperty
        {
            get { return (FuncDelegate)GetValue(FunctionPropertyProperty); }
            set { SetValue(FunctionPropertyProperty, value); }
        }

        // Using a DependencyProperty as the backing store for FunctionProperty.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty FunctionPropertyProperty =
            DependencyProperty.Register("FunctionProperty", typeof(FuncDelegate), typeof(MainWindow), new PropertyMetadata(null,PropertyChanged));



        private static void PropertyChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            MainWindow win = (MainWindow)d;
            DrawGraph(win);
        }

        private static void DrawGraph(MainWindow win)
        {
            win.DrawArea.Children.Clear();
            DrawAxes(win);
            DrawFunc(win);
        }

        private static void DrawAxes(MainWindow win)
        {
            double height = win.DrawArea.ActualHeight;
            double width = win.DrawArea.ActualWidth;
            double heigthHalf = height / 2;
            double widthHalf = width / 2;
            DrawLine(0, heigthHalf, width, heigthHalf, Brushes.Red, win.DrawArea);
            DrawLine(widthHalf, 0, widthHalf, height, Brushes.Red, win.DrawArea);
        }

        private static void DrawLine(double X1, double Y1, double X2, double Y2, SolidColorBrush color, Canvas canvas)
        {
            Line line = new Line
            {
                Stroke = color,
                X1 = X1,
                Y1 = Y1,
                X2 = X2,
                Y2 = Y2
            };
            canvas.Children.Add(line);
        }

        private static void DrawFunc(MainWindow win)
        {
            Polyline graph = new Polyline
            {
                Stroke = Brushes.Black
            };
            double width = win.DrawArea.ActualWidth,
                   height = win.DrawArea.ActualHeight;
            for(int i = 0; i < width; i++)
            {
                double X = ConvertX(i,win);
                double result = win.CoefAProperty * win.FunctionProperty(win.CoefBProperty * X);
                double Y = ConvertY(result,win);
                graph.Points.Add(new Point(i, Y));
            }
            win.DrawArea.Children.Add(graph);

        }

        private static double ConvertY(double result, MainWindow win)
        {
            double height = win.DrawArea.ActualHeight;
            double scale = height / 5.0;
            return height/2 - result * scale;
        }

        private static double ConvertX(int i,MainWindow win)
        {
            double width = win.DrawArea.ActualWidth;
            double step = 4 * Math.PI / width;
            return i * step - 2 * Math.PI;
        }

        public MainWindow()
        {
            InitializeComponent();
            FillCoefABox();
            FillFuncBox();
            FillCoefBBox();
        }

        private void FillCoefBBox()
        {
            for(int i = -3; i <= 3; i++)
            {
                if (i != 0)
                {
                    CoefBBox.Items.Add(i);
                }
            }
            CoefBBox.SelectedIndex = 3;
        }

        private void FillFuncBox()
        {
            ComboBoxItem sinItem = new ComboBoxItem
            {
                Tag = sinDelegate,
                Content = "Sin"
            },
                         cosItem = new ComboBoxItem();
            cosItem.Tag = cosDelegate;
            cosItem.Content = "Cos";
            FuncBox.Items.Add(sinItem);
            FuncBox.Items.Add(cosItem);
            FuncBox.SelectedIndex = 0;

        }

        private void FillCoefABox()
        {
            for(int i = -2; i <= 2; i++)
            {
                if (i != 0)
                {
                    CoefABox.Items.Add(i);
                }
            }
           // CoefABox.SelectedIndex = 2;
        }

        private void CoefABox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            CoefAProperty = (int)CoefABox.SelectedValue;
        }

        private void FuncBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            FunctionProperty = (FuncBox.SelectedIndex==0?sinDelegate:cosDelegate);
        }

        private void CoefBBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            CoefBProperty = (int)CoefBBox.SelectedValue;
        }

        private void SaveMenuItem_Click(object sender, RoutedEventArgs e)
        {
            SaveFileDialog dialog = new SaveFileDialog();
            if (dialog.ShowDialog() == true)
            {
                double width = DrawArea.ActualWidth;
                double height = DrawArea.ActualHeight;
                RenderTargetBitmap render = new RenderTargetBitmap((int)width, (int)height, 90, 90, PixelFormats.Default);
                render.Render(DrawArea);
                BitmapEncoder encoder = new PngBitmapEncoder();
                encoder.Frames.Add(BitmapFrame.Create(render));
                using (var fs = System.IO.File.OpenWrite(dialog.FileName))
                {
                    encoder.Save(fs);
                }
            }
        }

        private void ExitMenuItem_Click(object sender, RoutedEventArgs e)
        {
            Application.Current.Shutdown();
        }
    }
}
